﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Drawing;

namespace ELTE.Algorithms.BinarySearchTree.Serialize.Model
{
    /// <summary>
    /// Bináris keresőfa típusa.
    /// </summary>
    /// <typeparam name="KeyType">Kulcs típusa.</typeparam>
    /// <typeparam name="ValueType">Érték típusa.</typeparam>
    [Serializable]
    public partial class BinarySearchTree<KeyType, ValueType> where KeyType : IComparable<KeyType>
        // két sablonnal rendelkezik, a kulcséval, illetve az értékével
        // a kulcsra feltételt szabunk, hogy összehasonlíthatóó típusnak kell lennie
    {
        protected Node _Root;
        protected Node _Current;
        
        /// <summary>
        /// Fa gyökércsúcsának lekérdezése.
        /// </summary>
        public Node Root { get { return _Root; } }
        /// <summary>
        /// Aktuális csúcs lekérdezése.
        /// </summary>
        public Node Current { get { return _Current; } }
        /// <summary>
        /// Fa ürességének lekérdezése.
        /// </summary>
        public Boolean IsEmpty { get { return (_Root == null); } }

        /// <summary>
        /// Új bináris keresőfa létrehozása.
        /// </summary>
        public BinarySearchTree()
        {
            _Root = _Current = null;
        }

        /// <summary>
        /// Beszúrás.
        /// </summary>
        /// <param name="key">Kulcs.</param>
        /// <param name="value">Érték.</param>
        public void Insert(KeyType key, ValueType value)
        {
            Node y = null, x = _Root;
            while (x != null && !x.Key.Equals(key))
            {
                y = x;
                if (key.CompareTo(x.Key) < 0)
                    x = x.Left;
                else
                    x = x.Right;
            }
            if (x == null)
            {
                if (y == null)
                    _Root = _Current = new Node(key, value);
                else
                {
                    if (key.CompareTo(y.Key) < 0)
                        y.Left = _Current = new Node(key, value, y);
                    else
                        y.Right = _Current = new Node(key, value, y);
                }
            }
            else
                throw new DuplicateKeyException(); // kivétel dobása
        }
        /// <summary>
        /// Minimum elem kiválasztása.
        /// </summary>
        public void Min()
        {
            if (_Root == null)
                throw new TreeIsEmptyException();

            _Current = _Root;
            while (_Current.Left != null)
                _Current = _Current.Left;
        }
        /// <summary>
        /// Maximum elem kiválasztása.
        /// </summary>
        public void Max()
        {
            if (_Root == null)
                throw new TreeIsEmptyException();

            _Current = _Root;
            while (_Current.Right != null)
                _Current = _Current.Right;
        }
        /// <summary>
        /// Előző elem kiválassztása (ha van).
        /// </summary>
        public void Previous()
        {
            if (_Current != null)
            {
                if (_Current.Left != null)
                {
                    _Current = _Current.Left;
                    while (_Current.Right != null)
                        _Current = _Current.Right;
                }
                else
                {
                    Node p = _Current, s = _Current.Parent;
                    while (s != null && s.Left == p)
                    {
                        p = s;
                        s = s.Parent;
                    }
                    if (s != null)
                        _Current = s;
                }
            }
            else
                throw new TreeIsEmptyException();
        }
        /// <summary>
        /// Következő elem kiválassztása (ha van).
        /// </summary>
        public void Next()
        {
            if (_Current != null)
            {
                if (_Current.Right != null)
                {
                    _Current = _Current.Right;
                    while (_Current.Left != null)
                        _Current = _Current.Left;
                }
                else
                {
                    Node p = _Current, s = _Current.Parent;
                    while (s != null && s.Right == p)
                    {
                        p = s;
                        s = s.Parent;
                    }
                    if (s != null)
                        _Current = s;
                }
            }
            else
                throw new TreeIsEmptyException();
        }
        /// <summary>
        /// Keresés.
        /// </summary>
        /// <param name="key">Keresett kulcs.</param>
        /// <returns>Keresett elem értéke.</returns>
        public ValueType Search(KeyType key)
        {
            _Current = _Root;
            while (_Current != null && !_Current.Key.Equals(key))
            {
                if (key.CompareTo(_Current.Key) < 0)
                    _Current = _Current.Left;
                else
                    _Current = _Current.Right;
            }
            return _Current.Value;
        }
        /// <summary>
        /// Törlés.
        /// </summary>
        /// <param name="key">Törlendő kulcs.</param>
        public ValueType Delete(KeyType key)
        { 
            Node p = _Root;
            while (p != null && !p.Key.Equals(key))
            {
                if (key.CompareTo(p.Key) < 0)
                    p = p.Left;
                else
                    p = p.Right;
            }
            if (p == null)
                throw new KeyMissingException();

            Node x = null, y = null;

            if (p.Left == null || p.Right == null)
                y = p;
            else
            {
                y = p;
                y = y.Right;
                while (y.Left != null)
                    y = y.Left;
            }
            if (p != null)
            {
                if (y.Left != null)
                    x = y.Left;
                else
                    x = y.Right;
                if (x != null)
                    x.Parent = y.Parent;
                if (y.Parent == null)
                    _Root = x;
                else
                    if (y == y.Parent.Left)
                        y.Parent.Left = x;
                    else
                        y.Parent.Right = x;
                if (y != p)
                    p.Key = y.Key;
            }
            if (x != null)
                _Current = x;
            else
                _Current = y.Parent;
            return y.Value;
        }
        /// <summary>
        /// Fa kiürítése.
        /// </summary>
        public void Clear()
        {
            _Root = _Current = null;
        }
    }
}
