﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Text;
using System.Windows.Forms;

namespace ELTE.Forms.ImageDisplay
{
    public partial class MainForm : Form
    {
        private Bitmap _Image; // kép
        private Single _ZoomLevel; // közelítés mértéke

        public MainForm()
        {
            InitializeComponent();
            Text = Application.ProductName;
            MouseWheel += new MouseEventHandler(DisplayForm_MouseWheel);
            Resize += new EventHandler(DisplayForm_Resize);

            _OpenFileDialog.InitialDirectory = Environment.GetFolderPath(Environment.SpecialFolder.MyDocuments); // alapértelmezett könyvtár a dokumentumok

        }

        private void DisplayForm_Resize(object sender, EventArgs e) // ablak átméretezése
        {
            if (WindowState != FormWindowState.Minimized) // ha nem minimalizálva van
            {
                _ScrollHorizontal.Location = new Point(0, Height - 53);
                _ScrollHorizontal.Size = new Size(Width - 33, 17);
                _ScrollVertical.Location = new Point(Width - 33, 0);
                _ScrollVertical.Size = new Size(17, Height - 53);
                _PictureBoxMap.Size = new Size(Width - 33, Height - 81);
                DrawImage();
            }
        }

        private void DisplayForm_Load(object sender, EventArgs e)
        {
            _MenuFileClose.Enabled = _MenuView.Enabled = false;
            _MenuViewZoom100.Checked = true;
            _ZoomLevel = 1;
            RecalculateScrolls();
        }

        protected override void OnFormClosing(FormClosingEventArgs e) // kilépési eseménykezelő felüldefiniálása
        {
            if (e.CloseReason == CloseReason.UserClosing &&
                MessageBox.Show("Really exit?", "Exit " + Application.ProductName, MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.No)
                // megkérdezzük a felhasználót, hogy biztos be akarja-e zárni az ablakot
                e.Cancel = true; // ha igen, akkor töröljük az eseményt, így nem fog bezáródni az ablak
            base.OnFormClosing(e); // továbbadjuk a végrehajtást az ősnek
        }

        private void RecalculateScrolls() // scrollok kapcsolása és beállítása
        {
            if (_Image == null || _ZoomLevel == 0)
                _ScrollHorizontal.Enabled = _ScrollVertical.Enabled = false;
            else
            {
                if (_PictureBoxMap.Width < _Image.Width * _ZoomLevel)
                {
                    _ScrollHorizontal.Enabled = true;
                    _ScrollHorizontal.Maximum = Convert.ToInt32(_Image.Width * _ZoomLevel - _PictureBoxMap.Width);

                }
                else
                    _ScrollHorizontal.Enabled = false;                    
                if (_PictureBoxMap.Height < _Image.Height * _ZoomLevel)
                {
                    _ScrollVertical.Enabled = true;
                    _ScrollVertical.Maximum = Convert.ToInt32(_Image.Height * _ZoomLevel - _PictureBoxMap.Height);
                }
                else
                    _ScrollVertical.Enabled = false;
            }
        }
        
        private void DrawImage() // kép kirajzolása
        {
            RecalculateScrolls();
            if (_Image != null)
            {
                if (_PictureBoxMap.Image != null)
                    _PictureBoxMap.Image.Dispose(); // korábbi kép törlése
                if (_ZoomLevel == 0) // beillesztés a képbe
                {
                    _PictureBoxMap.SizeMode = PictureBoxSizeMode.Zoom; // beállítjuk a képmegjelenítőnek, hogy zoomoljon
                    _PictureBoxMap.Image = (_Image.Clone() as Bitmap); // leklónozzuk a képet, ekkor keletkezik egy másodpéldány
                }
                else // kivágás
                {
                    _PictureBoxMap.SizeMode = PictureBoxSizeMode.Normal; // a képmegjelenítő eredi méretben jelenít meg a képet

                    // új kép, a zommoláshoz mérten:
                    Bitmap newImage = new Bitmap(Convert.ToInt32(_Image.Width * _ZoomLevel), Convert.ToInt32(_Image.Height * _ZoomLevel));
                    Graphics graphics = Graphics.FromImage(newImage);
                    graphics.InterpolationMode = InterpolationMode.HighQualityBicubic; // a zoomoláshoz interpolálunk

                    graphics.DrawImage(_Image, 0, 0, newImage.Width, newImage.Height);
                    graphics.Dispose();

                    // megfelelő méretű kép kivágása:
                    Int32 startX = Convert.ToInt32(_ScrollHorizontal.Value); 
                    Int32 startY = Convert.ToInt32(_ScrollVertical.Value); 
                    _PictureBoxMap.Image = newImage.Clone(new Rectangle(startX, startY,
                                                                        (_PictureBoxMap.Width < newImage.Width - startX) ? _PictureBoxMap.Width : newImage.Width - startX,
                                                                        (_PictureBoxMap.Height < newImage.Height - startY) ? _PictureBoxMap.Height : newImage.Height - startY),
                                                                        newImage.PixelFormat);
                    newImage.Dispose();
                }
            }
        }

        private void DisplayForm_MouseWheel(object sender, MouseEventArgs e)  // egérgörgő mozgatás
        {
            if (_ScrollVertical.Value - e.Delta / 100 > _ScrollVertical.Minimum && _ScrollVertical.Value - e.Delta / 100 < _ScrollVertical.Maximum)
                _ScrollVertical.Value -= e.Delta / 100;
            DrawImage();
        }

        // scrollozás
        private void ScrollVertical_Scroll(object sender, ScrollEventArgs e)
        {
            DrawImage();
        }

        private void ScrollHorizontal_Scroll(object sender, ScrollEventArgs e)
        {
            DrawImage();
        }


        private void MenuFileOpen_Click(object sender, EventArgs e) // fájl betöltése menü
        {
            if (_OpenFileDialog.ShowDialog() == DialogResult.OK)
            {
                _Image = new Bitmap(_OpenFileDialog.FileName);
                DrawImage();
                _MenuFileClose.Enabled = _MenuView.Enabled = true;
                RecalculateScrolls();
            }
        }

        private void MenuFileClose_Click(object sender, EventArgs e) // kép bezárása menü
        {
            _PictureBoxMap.Image = null;
            if (_Image != null)
                _Image.Dispose();
            _Image = null;
            _MenuFileClose.Enabled = _MenuView.Enabled = false;
        }
       
        private void MenuFileExit_Click(object sender, EventArgs e)
        {
            Close();
        }
        
        private void MenuViewZoom100_Click(object sender, EventArgs e)
        {
            _MenuViewZoom10.Checked = _MenuViewZoom200.Checked =
            _MenuViewZoom50.Checked = _MenuViewZoom500.Checked = 
            _MenuViewFitIn.Checked = false; // állítjuk a menüpontok kijelölését

            _MenuViewZoom100.Checked = true;
            _ZoomLevel = 1;

            DrawImage();
        }
        private void MenuViewZoom10_Click(object sender, EventArgs e)
        {
            _MenuViewZoom100.Checked = _MenuViewZoom200.Checked =
            _MenuViewZoom50.Checked = _MenuViewZoom500.Checked =
            _MenuViewFitIn.Checked = false;

            _MenuViewZoom10.Checked = true;
            _ZoomLevel = Convert.ToSingle(Math.Sqrt(0.1));

            DrawImage();
        }
        private void MenuViewZoom50_Click(object sender, EventArgs e)
        {
            _MenuViewZoom10.Checked = _MenuViewZoom200.Checked =
            _MenuViewZoom100.Checked = _MenuViewZoom500.Checked =
            _MenuViewFitIn.Checked = false;

            _MenuViewZoom50.Checked = true;
            _ZoomLevel = Convert.ToSingle(Math.Sqrt(0.5));

            DrawImage();
        }
        private void MenuViewZoom200_Click(object sender, EventArgs e)
        {
            _MenuViewZoom10.Checked = _MenuViewZoom100.Checked =
            _MenuViewZoom50.Checked = _MenuViewZoom500.Checked =
            _MenuViewFitIn.Checked = false;

            _MenuViewZoom200.Checked = true;
            _ZoomLevel = Convert.ToSingle(Math.Sqrt(2));

            DrawImage();
        }
        private void MenuViewZoom500_Click(object sender, EventArgs e)
        {
            _MenuViewZoom10.Checked = _MenuViewZoom200.Checked =
            _MenuViewZoom50.Checked = _MenuViewZoom100.Checked =
            _MenuViewFitIn.Checked = false;

            _MenuViewZoom500.Checked = true;
            _ZoomLevel = Convert.ToSingle(Math.Sqrt(5));

            DrawImage();
        }
        private void MenuViewFitIn_Click(object sender, EventArgs e)
        {
            _MenuViewZoom10.Checked = _MenuViewZoom100.Checked =
            _MenuViewZoom50.Checked = _MenuViewZoom200.Checked =
            _MenuViewZoom500.Checked = false;

            _MenuViewFitIn.Checked = true;
            _ZoomLevel = 0;

            DrawImage();
        }

    }
}
