﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Drawing;
using System.Windows.Forms;
using System.IO;

namespace ELTE.Forms.MultiImageDisplay.Model
{
    /// <summary>
    /// Megjeleníthető kép típusa. 
    /// </summary>
    public class DisplayableImage
    {
        private Image _MapImage; // raszterkép
        private String _Title; // neve
        private Double _ZoomLevel; //közelítés mértéke
        private PictureBox _ZoomDisplay; // közelítést megjelenítő képdoboz
        private PictureBox _FullDisplay; // teljes képet megjelenítő képbodoz
        private Int32 _CenterX; // közelítés középpontja
        private Int32 _CenterY;
        private Int32 _Step; // mozgatás mértéke

        /// <summary>
        /// A képhez nagyított részéhez tartozó képmegjelenítő lekérdezése, vagy beállítása.
        /// </summary>
        public PictureBox ZoomDisplay
        {
            set { _ZoomDisplay = value; Draw(); }
            get { return _ZoomDisplay; }
        }
        /// <summary>
        /// A képet teljes egészében tartalmazó képmegjelenítő lekérdezése, vagy beállítása.
        /// </summary>
        public PictureBox FullDisplay
        {
            set
            {
                _FullDisplay = value;
                Draw();
            }
            get { return _FullDisplay; }
        }
        /// <summary>
        /// A nagyítás mértékének lekérdezése, vagy beállítása.
        /// </summary>
        public Int32 ZoomLevel
        {
            set { _ZoomLevel = Convert.ToSingle(Math.Sqrt(Convert.ToSingle(value) / 100)); Draw(); }
            get { return Convert.ToInt32(_ZoomLevel * 100); }
        }
        /// <summary>
        /// A képi objektum lekérdezése.
        /// </summary>
        public Image Image
        {
            get { return _MapImage; }
        }
        /// <summary>
        /// A kép címének lekérdezése.
        /// </summary>
        public String Title
        {
            get { return _Title; }
        }

        /// <summary>
        /// Megjeleníthető kép létrehozása fájlnév alapján.
        /// </summary>
        /// <param name="fileName">Fájlnév.</param>
        public DisplayableImage(String fileName)
        {
            FileInfo info = new FileInfo(fileName); // fájl információs objektum, amelyet a fájlnév lekérdezésére használunk
            _MapImage = new Bitmap(fileName);
            _Title = info.Name; // a kép címét a fájlnév alapján állapítjuk meg
            _Step = (_MapImage.Width < _MapImage.Height) ? _MapImage.Width / 10 : _MapImage.Height / 10;
            Reset();
        }

        /// <summary>
        /// Nagyítás és pozícionálás visszaállítása.
        /// </summary>
        public void Reset()
        {
            _ZoomLevel = 1;
            _CenterX = _MapImage.Width / 2;
            _CenterY = _MapImage.Height / 2;
            Draw();
        }

        /// <summary>
        /// Középpont mozgatása balra.
        /// </summary>
        public void Left()
        {
            if (_ZoomDisplay != null)
            {
                if (_CenterX - _ZoomDisplay.Width / 2 >= _Step / _ZoomLevel)
                    _CenterX -= Convert.ToInt32(_Step / _ZoomLevel);
                else
                    _CenterX = Convert.ToInt32(_ZoomDisplay.Width / 2 / _ZoomLevel);
            }
            Draw();
        }
        /// <summary>
        /// Középpont mozgatása jobbra.
        /// </summary>
        public void Right()
        {
            if (_ZoomDisplay != null)
            {
                if (_CenterX + _ZoomDisplay.Width / 2 / _ZoomLevel < _MapImage.Width - _Step / _ZoomLevel)
                    _CenterX += Convert.ToInt32(_Step / _ZoomLevel);
                else
                    _CenterX = Convert.ToInt32(_MapImage.Width - Math.Ceiling(_ZoomDisplay.Width / 2 / _ZoomLevel));
            }
            Draw();
        }
        /// <summary>
        /// Középpont mozgatása felfelé.
        /// </summary>
        public void Up()
        {
            if (_ZoomDisplay != null)
            {
                if (_CenterY - _ZoomDisplay.Height / 2 >= _Step / _ZoomLevel)
                    _CenterY -= Convert.ToInt32(_Step / _ZoomLevel);
                else
                    _CenterY = Convert.ToInt32(_ZoomDisplay.Height / 2 / _ZoomLevel);
            }
            Draw();
        }
        /// <summary>
        /// Középpont mozgatása lefelé.
        /// </summary>
        public void Down()
        {
            if (_ZoomDisplay != null)
            {
                if (_CenterY + _ZoomDisplay.Height / 2 / _ZoomLevel < _MapImage.Height - _Step / _ZoomLevel)
                    _CenterY += Convert.ToInt32(_Step / _ZoomLevel);
                else
                    _CenterY = Convert.ToInt32(_MapImage.Height - Math.Ceiling(_ZoomDisplay.Height / 2 / _ZoomLevel));
            }
            Draw();
        }
        /// <summary>
        /// Középpont mozgatása adott pozícióra.
        /// </summary>
        /// <param name="x">Vízszintes pozíció.</param>
        /// <param name="y">Függőleges pozíció.</param>
        public void Move(Int32 x, Int32 y)
        {
            if (_ZoomDisplay != null)
            {
                if ((_CenterX * _ZoomLevel + x - _ZoomDisplay.Width / 2) >= 0 && (_CenterX * _ZoomLevel + x + _ZoomDisplay.Width / 2) < _MapImage.Width * _ZoomLevel)
                    _CenterX += x;
                if ((_CenterY * _ZoomLevel + y - _ZoomDisplay.Height / 2) >= 0 && (_CenterY * _ZoomLevel + y + _ZoomDisplay.Height / 2) < _MapImage.Height * _ZoomLevel)
                    _CenterY += y;
            }
            Draw();
        }

        /// <summary>
        /// Kép kirajzolása.
        /// </summary>
        private void Draw()
        {
            if (_ZoomDisplay != null)
            {
                if (_ZoomDisplay.Image != null)
                    _ZoomDisplay.Image.Dispose();

                // létrehozunk egy új képet a nagyított nézethez:
                Bitmap zoomedImage = new Bitmap((Int32)(Math.Ceiling(_MapImage.Width * _ZoomLevel)), (Int32)(Math.Ceiling(_MapImage.Height * _ZoomLevel)));

                // a rajzolást nem a képernyőn végezzük, hanem a háttérben:
                Graphics zoomedImageGraphics = Graphics.FromImage((Image)zoomedImage);

                // zoomoláshoz interpoláció:
                zoomedImageGraphics.InterpolationMode = System.Drawing.Drawing2D.InterpolationMode.HighQualityBicubic; // a zoomoláshoz interpolálunk
                zoomedImageGraphics.DrawImage(_MapImage, 0, 0, zoomedImage.Width, zoomedImage.Height);

                // megfelelő képrészlet kivágása:
                Int32 startX = (_CenterX * _ZoomLevel < _ZoomDisplay.Width / 2) ? 0 : Convert.ToInt32(_CenterX * _ZoomLevel - _ZoomDisplay.Width / 2);
                Int32 startY = (_CenterY * _ZoomLevel < _ZoomDisplay.Height / 2) ? 0 : Convert.ToInt32(_CenterY * _ZoomLevel - _ZoomDisplay.Height / 2);
                Int32 sizeX = (startX + _ZoomDisplay.Width < zoomedImage.Width) ? _ZoomDisplay.Width : zoomedImage.Width - startX;
                Int32 sizeY = (startY + _ZoomDisplay.Height < zoomedImage.Height) ? _ZoomDisplay.Height : zoomedImage.Height - startY;

                _ZoomDisplay.Image = zoomedImage.Clone(new Rectangle(startX, startY, sizeX, sizeY), zoomedImage.PixelFormat);
                zoomedImage.Dispose();

                if (_FullDisplay != null)
                {
                    // méretarányok meghatározása:
                    Double scale = ((Double)_MapImage.Width / _FullDisplay.Width > (Double)_MapImage.Height / _FullDisplay.Height) ? (Double)_MapImage.Width / _FullDisplay.Width : (Double)_MapImage.Height / _FullDisplay.Height;
                    Int32 tStartX = Convert.ToInt32(_FullDisplay.Width - Math.Ceiling(_MapImage.Width / scale)) / 2;
                    Int32 tStartY = Convert.ToInt32(_FullDisplay.Height - Math.Ceiling(_MapImage.Height / scale)) / 2;
                    Int32 tSizeX = Convert.ToInt32(_MapImage.Width / scale);
                    Int32 tSizeY = Convert.ToInt32(_MapImage.Height / scale);

                    // új kép létrehozása a kijelölőnégyzethez:
                    Image fullImage = new Bitmap(_FullDisplay.Width, _FullDisplay.Height);

                    // a rajzolást nem a képernyőn végezzük, hanem a háttérben:
                    Graphics fullImageGraphics = Graphics.FromImage(fullImage);
                    fullImageGraphics.FillRectangle(Brushes.White, 0, 0, _FullDisplay.Width, _FullDisplay.Height);
                    fullImageGraphics.DrawImage(_MapImage, tStartX, tStartY, tSizeX, tSizeY);
                    // kijelölőnégyzet rajzolása:
                    fullImageGraphics.DrawRectangle(new Pen(Color.Red),
                                     Convert.ToSingle(tStartX + startX / scale / _ZoomLevel),
                                     Convert.ToSingle(tStartY + startY / scale / _ZoomLevel),
                                     Convert.ToSingle(sizeX / _ZoomLevel / scale - 1),
                                     Convert.ToSingle(sizeY / _ZoomLevel / scale - 1));

                    // kitesszük a képet:
                    _FullDisplay.CreateGraphics().DrawImage(fullImage, 0, 0);
                }
            }
        }
    }
}
